@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2011-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2018 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -16,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -57,6 +58,8 @@ struct gcm128_context {
 struct aes_gcm_ctx {
 	AES_KEY expanded_key;
 	struct gcm128_context gcm;
+	unsigned finished;
+	unsigned auth_finished;
 };
 
 void gcm_init_clmul(u128 Htable[16], const uint64_t Xi[2]);
@@ -134,6 +137,8 @@ static int aes_gcm_setiv(void *_ctx, const void *iv, size_t iv_size)
 	aesni_ecb_encrypt(ctx->gcm.Yi.c, ctx->gcm.EK0.c,
 			  GCM_BLOCK_SIZE, ALIGN16(&ctx->expanded_key), 1);
 	ctx->gcm.Yi.c[GCM_BLOCK_SIZE - 1] = 2;
+	ctx->finished = 0;
+	ctx->auth_finished = 0;
 	return 0;
 }
 
@@ -179,6 +184,9 @@ aes_gcm_encrypt(void *_ctx, const void *src, size_t src_size,
 	int rest = src_size - (exp_blocks);
 	uint32_t counter;
 
+	if (unlikely(ctx->finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	if (blocks > 0) {
 		aesni_ctr32_encrypt_blocks(src, dst,
 					   blocks,
@@ -190,8 +198,10 @@ aes_gcm_encrypt(void *_ctx, const void *src, size_t src_size,
 		_gnutls_write_uint32(counter, ctx->gcm.Yi.c + 12);
 	}
 
-	if (rest > 0)		/* last incomplete block */
+	if (rest > 0) {	/* last incomplete block */
 		ctr_encrypt_last(ctx, src, dst, exp_blocks, rest);
+		ctx->finished = 1;
+	}
 
 	gcm_ghash(ctx, dst, src_size);
 	ctx->gcm.len.u[1] += src_size;
@@ -209,6 +219,9 @@ aes_gcm_decrypt(void *_ctx, const void *src, size_t src_size,
 	int rest = src_size - (exp_blocks);
 	uint32_t counter;
 
+	if (unlikely(ctx->finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	gcm_ghash(ctx, src, src_size);
 	ctx->gcm.len.u[1] += src_size;
 
@@ -223,8 +236,10 @@ aes_gcm_decrypt(void *_ctx, const void *src, size_t src_size,
 		_gnutls_write_uint32(counter, ctx->gcm.Yi.c + 12);
 	}
 
-	if (rest > 0)		/* last incomplete block */
+	if (rest > 0) {	/* last incomplete block */
 		ctr_encrypt_last(ctx, src, dst, exp_blocks, rest);
+		ctx->finished = 1;
+	}
 
 	return 0;
 }
@@ -233,9 +248,15 @@ static int aes_gcm_auth(void *_ctx, const void *src, size_t src_size)
 {
 	struct aes_gcm_ctx *ctx = _ctx;
 
+	if (unlikely(ctx->auth_finished))
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	gcm_ghash(ctx, src, src_size);
 	ctx->gcm.len.u[0] += src_size;
 
+	if (src_size % GCM_BLOCK_SIZE != 0)
+		ctx->auth_finished = 1;
+
 	return 0;
 }
 
