@@ -27,12 +27,13 @@
 #include "errors.h"
 #include "num.h"
 #include "hello_ext_lib.h"
-#include "ext/cookie.h"
+#include <ext/cookie.h>
 
-static int cookie_recv_params(gnutls_session_t session, const uint8_t *data,
-			      size_t data_size);
+static int cookie_recv_params(gnutls_session_t session,
+					  const uint8_t * data,
+					  size_t data_size);
 static int cookie_send_params(gnutls_session_t session,
-			      gnutls_buffer_st *extdata);
+					  gnutls_buffer_st * extdata);
 
 const hello_ext_entry_st ext_mod_cookie = {
 	.name = "Cookie",
@@ -40,10 +41,7 @@ const hello_ext_entry_st ext_mod_cookie = {
 	.gid = GNUTLS_EXTENSION_COOKIE,
 	.validity = GNUTLS_EXT_FLAG_TLS | GNUTLS_EXT_FLAG_CLIENT_HELLO |
 		    GNUTLS_EXT_FLAG_HRR | GNUTLS_EXT_FLAG_IGNORE_CLIENT_REQUEST,
-	.client_parse_point =
-		GNUTLS_EXT_MANDATORY, /* force parsing prior to EXT_TLS extensions */
-	.server_parse_point =
-		GNUTLS_EXT_MANDATORY, /* force parsing prior to EXT_TLS extensions */
+	.parse_type = GNUTLS_EXT_MANDATORY, /* force parsing prior to EXT_TLS extensions */
 	.recv_func = cookie_recv_params,
 	.send_func = cookie_send_params,
 	.pack_func = NULL,
@@ -53,10 +51,12 @@ const hello_ext_entry_st ext_mod_cookie = {
 };
 
 /* Only client sends this extension. */
-static int cookie_recv_params(gnutls_session_t session, const uint8_t *data,
-			      size_t data_size)
+static int
+cookie_recv_params(gnutls_session_t session,
+		   const uint8_t * data, size_t _data_size)
 {
-	size_t csize;
+	ssize_t data_size = _data_size;
+	ssize_t csize;
 	int ret;
 	gnutls_datum_t tmp;
 
@@ -73,14 +73,12 @@ static int cookie_recv_params(gnutls_session_t session, const uint8_t *data,
 			DECR_LEN(data_size, csize);
 
 			if (data_size != 0)
-				return gnutls_assert_val(
-					GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
+				return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-			tmp.data = (void *)data;
+			tmp.data = (void*)data;
 			tmp.size = csize;
 
-			ret = _gnutls_hello_ext_set_datum(
-				session, GNUTLS_EXTENSION_COOKIE, &tmp);
+			ret = _gnutls_hello_ext_set_datum(session, GNUTLS_EXTENSION_COOKIE, &tmp);
 			if (ret < 0)
 				return gnutls_assert_val(ret);
 
@@ -95,21 +93,20 @@ static int cookie_recv_params(gnutls_session_t session, const uint8_t *data,
 
 /* returns data_size or a negative number on failure
  */
-static int cookie_send_params(gnutls_session_t session,
-			      gnutls_buffer_st *extdata)
+static int
+cookie_send_params(gnutls_session_t session,
+		   gnutls_buffer_st * extdata)
 {
 	gnutls_datum_t tmp;
 	int ret;
 
 	/* this function sends the client extension data (dnsname) */
 	if (session->security_parameters.entity == GNUTLS_CLIENT) {
-		ret = _gnutls_hello_ext_get_datum(
-			session, GNUTLS_EXTENSION_COOKIE, &tmp);
+		ret = _gnutls_hello_ext_get_datum(session, GNUTLS_EXTENSION_COOKIE, &tmp);
 		if (ret < 0)
 			return 0;
 
-		ret = _gnutls_buffer_append_data_prefix(extdata, 16, tmp.data,
-							tmp.size);
+		ret = _gnutls_buffer_append_data_prefix(extdata, 16, tmp.data, tmp.size);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 
