@@ -48,8 +48,7 @@ const hello_ext_entry_st ext_mod_server_cert_type = {
 	.name = "Server Certificate Type",
 	.tls_id = 20,
 	.gid = GNUTLS_EXTENSION_SERVER_CERT_TYPE,
-	.client_parse_point = GNUTLS_EXT_TLS,
-	.server_parse_point = GNUTLS_EXT_TLS,
+	.parse_type = GNUTLS_EXT_TLS,
 	.validity = GNUTLS_EXT_FLAG_TLS |
 		GNUTLS_EXT_FLAG_DTLS |
 		GNUTLS_EXT_FLAG_CLIENT_HELLO |
@@ -69,9 +68,12 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 						size_t data_size)
 {
 	int ret;
+	gnutls_datum_t cert_types; // Holds the received cert types
+	gnutls_datum_t sent_cert_types; // Holds the previously sent cert types
 	gnutls_certificate_type_t cert_type;
-	size_t i;
-	bool found = false;
+
+	uint8_t i, found = 0;
+	ssize_t len = data_size;
 	const uint8_t* pdata = data;
 
 	/* Only activate this extension if we have cert credentials set
@@ -81,11 +83,10 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 		return 0;
 
 	if (!IS_SERVER(session)) {	// client mode
-		gnutls_datum_t sent_cert_types; // Holds the previously sent cert types
 
 		/* Compare packet length with expected packet length. For the
 		 * client this is a single byte. */
-		if (data_size != 1) {
+		if (len != 1) {
 			return
 					gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 		}
@@ -97,9 +98,6 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 		 * we're going to check it nevertheless. */
 		cert_type = IANA2cert_type(pdata[0]);
 
-		_gnutls_handshake_log("EXT[%p]: Received a %s server certificate type confirmation from the server.\n",
-		                      session, gnutls_certificate_type_get_name(cert_type));
-
 		// Check validity of cert type
 		if (cert_type == GNUTLS_CRT_UNKNOWN) {
 			return gnutls_assert_val(GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE);
@@ -121,7 +119,7 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 		// Check whether what we got back is actually offered by us
 		for (i = 0; i < sent_cert_types.size; i++) {
 			if (IANA2cert_type(sent_cert_types.data[i]) == cert_type)
-				found = true;
+				found = 1;
 		}
 
 		if (found) {
@@ -136,11 +134,9 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 		return ret;
 
 	} else {		// server mode
-		gnutls_datum_t cert_types; // Holds the received cert types
-
 		// Compare packet length with expected packet length.
-		DECR_LEN(data_size, 1);
-		if (data[0] != data_size) {
+		DECR_LEN(len, 1);
+		if (data[0] != len) {
 			return
 					gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 		}
@@ -148,7 +144,7 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 
 		// Assign the contents of our data buffer to a gnutls_datum_t
 		cert_types.data = (uint8_t*)pdata; // Need casting to get rid of 'discards const qualifier' warning
-		cert_types.size = data_size;
+		cert_types.size = len;
 
 		// Store the server certificate types in our session
 		_gnutls_hello_ext_set_datum(session,
@@ -169,12 +165,10 @@ static int _gnutls_server_cert_type_recv_params(gnutls_session_t session,
 			if (cert_type == GNUTLS_CRT_UNKNOWN)
 				continue;
 
-			_gnutls_handshake_log("EXT[%p]: Checking compatibility of a %s server certificate type that was received from the client.\n",
-			                      session, gnutls_certificate_type_get_name(cert_type));
-
 			// Check for support of this cert type
-			if (_gnutls_session_is_cert_type_supported(session, cert_type, true, GNUTLS_CTYPE_SERVER) == 0) {
-				found = true;
+			if (_gnutls_session_cert_type_supported
+					(session, cert_type, true, GNUTLS_CTYPE_SERVER) == 0) {
+				found = 1;
 				break;
 			}
 		}
@@ -199,23 +193,22 @@ static int _gnutls_server_cert_type_send_params(gnutls_session_t session,
 						gnutls_buffer_st* data)
 {
 	int ret;
-	uint8_t cert_type_IANA; // Holds an IANA cert type ID
-	gnutls_certificate_type_t cert_type;
+	uint8_t cert_type; // Holds an IANA cert type ID
+	uint8_t i = 0, num_cert_types = 0;
+	priority_st* cert_priorities;
+	gnutls_datum_t tmp_cert_types; // For type conversion
+	uint8_t cert_types[GNUTLS_CRT_MAX]; // The list with supported cert types. Inv: 0 <= cert type Id < 256
 
 	/* Only activate this extension if we have cert credentials set
 	 * and alternative cert types are allowed */
 	if (!are_alternative_cert_types_allowed(session) ||
-	   (_gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE) == NULL))
+		(_gnutls_get_cred(session, GNUTLS_CRD_CERTIFICATE) == NULL))
 		return 0;
 
 	if (!IS_SERVER(session)) {	// Client mode
-		uint8_t cert_types[GNUTLS_CRT_MAX]; // The list with supported (IANA) cert types. Inv: 0 <= cert type Id < 256
-		size_t i, num_cert_types = 0;
-		priority_st* cert_priorities;
-		gnutls_datum_t tmp_cert_types; // For type conversion
-
 		// For brevity
-		cert_priorities = &session->internals.priorities->server_ctype;
+		cert_priorities =
+				&session->internals.priorities->server_ctype;
 
 		/* Retrieve server certificate type priorities if any. If no
 		 * priorities are set then the default server certificate type
@@ -251,11 +244,9 @@ static int _gnutls_server_cert_type_send_params(gnutls_session_t session,
 			 * structure of the code here.
 			 */
 			for (i = 0; i < cert_priorities->num_priorities; i++) {
-
-				cert_type = cert_priorities->priorities[i];
-
-				if (_gnutls_session_is_cert_type_supported(session, cert_type,
-				    false, GNUTLS_CTYPE_SERVER) == 0) {
+				if (_gnutls_session_cert_type_supported
+						(session, cert_priorities->priorities[i],
+						 false, GNUTLS_CTYPE_SERVER) == 0) {
 					/* Check whether we are allowed to store another cert type
 					 * in our buffer. In other words, prevent a possible buffer
 					 * overflow. This situation can occur when a user sets
@@ -264,22 +255,22 @@ static int _gnutls_server_cert_type_send_params(gnutls_session_t session,
 						return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
 
 					// Convert to IANA representation
-					ret = cert_type2IANA(cert_type);
+					ret = cert_type2IANA(cert_priorities->priorities[i]);
 
 					if (ret < 0)
 						return gnutls_assert_val(ret);
 
-					cert_type_IANA = ret; // For readability
+					cert_type = ret; // For readability
 
 					// Add this cert type to our list with supported types
-					cert_types[num_cert_types] = cert_type_IANA;
+					cert_types[num_cert_types] = cert_type;
 					num_cert_types++;
 
 					_gnutls_handshake_log
 							("EXT[%p]: Server certificate type %s (%d) was queued.\n",
 							 session,
-							 gnutls_certificate_type_get_name(cert_type),
-							 cert_type_IANA);
+							 gnutls_certificate_type_get_name(cert_priorities->priorities[i]),
+							 cert_type);
 				}
 			}
 
@@ -335,18 +326,15 @@ static int _gnutls_server_cert_type_send_params(gnutls_session_t session,
 		}
 	} else {	// Server mode
 		// Retrieve negotiated server certificate type and send it
-		cert_type = get_certificate_type(session, GNUTLS_CTYPE_SERVER);
-		ret = cert_type2IANA(cert_type);
+		ret = cert_type2IANA(get_certificate_type(
+					session, GNUTLS_CTYPE_SERVER));
 
 		if (ret < 0)
 			return gnutls_assert_val(ret);
 
-		cert_type_IANA = ret; // For readability
+		cert_type = ret; // For readability
 
-		_gnutls_handshake_log("EXT[%p]: Confirming to use a %s server certificate type.\n",
-		                      session, gnutls_certificate_type_get_name(cert_type));
-
-		ret = gnutls_buffer_append_data(data, &cert_type_IANA, 1);
+		ret = gnutls_buffer_append_data(data, &cert_type, 1);
 
 		if (ret < 0)
 			return gnutls_assert_val(ret);
