@@ -17,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -28,7 +28,7 @@
 #include <random.h>
 #include <gnutls/pkcs11.h>
 
-#include <extensions.h>	/* for _gnutls_ext_init */
+#include <hello_ext.h>	/* for _gnutls_hello_ext_init */
 #include <supplemental.h> /* for _gnutls_supplemental_deinit */
 #include <locks.h>
 #include <system.h>
@@ -310,7 +310,7 @@ static int _gnutls_global_init(unsigned constructor)
 	}
 
 	/* Initialize the default TLS extensions */
-	ret = _gnutls_ext_init();
+	ret = _gnutls_hello_ext_init();
 	if (ret < 0) {
 		gnutls_assert();
 		goto out;
@@ -369,8 +369,8 @@ static int _gnutls_global_init(unsigned constructor)
 	_gnutls_load_system_priorities();
 
 #ifdef ENABLE_FIPS140
-	/* These self tests are performed on the overriden algorithms
-	 * (e.g., AESNI overriden AES). They are after _gnutls_register_accel_crypto()
+	/* These self tests are performed on the overridden algorithms
+	 * (e.g., AESNI overridden AES). They are after _gnutls_register_accel_crypto()
 	 * intentionally */
 	if (res != 0) {
 		ret = _gnutls_fips_perform_self_checks2();
@@ -412,7 +412,7 @@ static void _gnutls_global_deinit(unsigned destructor)
 		_gnutls_system_key_deinit();
 		gnutls_crypto_deinit();
 		_gnutls_rnd_deinit();
-		_gnutls_ext_deinit();
+		_gnutls_hello_ext_deinit();
 		asn1_delete_structure(&_gnutls_gnutls_asn);
 		asn1_delete_structure(&_gnutls_pkix1_asn);
 
@@ -436,6 +436,8 @@ static void _gnutls_global_deinit(unsigned destructor)
 		_gnutls_tpm_global_deinit();
 #endif
 
+		_gnutls_nss_keylog_deinit();
+
 		gnutls_mutex_deinit(&_gnutls_file_mutex);
 		gnutls_mutex_deinit(&_gnutls_pkcs11_mutex);
 	} else {
