@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2000-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2017 Red Hat, Inc.
  *
  * Author: Nikos Mavrogiannopoulos
  *
@@ -16,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -38,11 +39,12 @@
 #include <session_pack.h>
 #include <datum.h>
 #include <num.h>
-#include <extensions.h>
+#include <hello_ext.h>
 #include <constate.h>
 #include <algorithms.h>
 #include <state.h>
 #include <db.h>
+#include "tls13/session_ticket.h"
 
 static int pack_certificate_auth_info(gnutls_session_t,
 				      gnutls_buffer_st * packed_session);
@@ -68,6 +70,10 @@ static int unpack_security_parameters(gnutls_session_t session,
 				      gnutls_buffer_st * packed_session);
 static int pack_security_parameters(gnutls_session_t session,
 				    gnutls_buffer_st * packed_session);
+static int tls13_unpack_security_parameters(gnutls_session_t session,
+					    gnutls_buffer_st * packed_session);
+static int tls13_pack_security_parameters(gnutls_session_t session,
+					  gnutls_buffer_st * packed_session);
 
 
 /* Since auth_info structures contain malloced data, this function
@@ -98,6 +104,7 @@ _gnutls_session_pack(gnutls_session_t session,
 
 	BUFFER_APPEND_NUM(&sb, PACKED_SESSION_MAGIC);
 	BUFFER_APPEND_NUM(&sb, session->security_parameters.timestamp);
+	BUFFER_APPEND_NUM(&sb, session->internals.expire_time);
 	BUFFER_APPEND(&sb, &id, 1);
 
 	switch (id) {
@@ -149,10 +156,22 @@ _gnutls_session_pack(gnutls_session_t session,
 		goto fail;
 	}
 
-	ret = _gnutls_ext_pack(session, &sb);
-	if (ret < 0) {
-		gnutls_assert();
-		goto fail;
+
+	if (session->security_parameters.pversion->tls13_sem) {
+		ret = tls13_pack_security_parameters(session, &sb);
+		if (ret < 0) {
+			gnutls_assert();
+			goto fail;
+		}
+	}
+
+	/* Extensions are re-negotiated in a resumed session under TLS 1.3 */
+	if (!session->security_parameters.pversion->tls13_sem) {
+		ret = _gnutls_hello_ext_pack(session, &sb);
+		if (ret < 0) {
+			gnutls_assert();
+			goto fail;
+		}
 	}
 
 	return _gnutls_buffer_to_datum(&sb, packed_session, 0);
@@ -172,6 +191,7 @@ _gnutls_session_unpack(gnutls_session_t session,
 	int ret;
 	gnutls_buffer_st sb;
 	uint32_t magic;
+	uint32_t expire_time;
 	uint8_t id;
 
 	_gnutls_buffer_init(&sb);
@@ -202,6 +222,8 @@ _gnutls_session_unpack(gnutls_session_t session,
 	BUFFER_POP_NUM(&sb,
 		       session->internals.resumed_security_parameters.
 		       timestamp);
+	BUFFER_POP_NUM(&sb, expire_time);
+	(void) expire_time;
 	BUFFER_POP(&sb, &id, 1);
 
 	switch (id) {
@@ -255,10 +277,21 @@ _gnutls_session_unpack(gnutls_session_t session,
 		goto error;
 	}
 
-	ret = _gnutls_ext_unpack(session, &sb);
-	if (ret < 0) {
-		gnutls_assert();
-		goto error;
+	if (session->internals.resumed_security_parameters.pversion->tls13_sem) {
+		/* 'prf' will not be NULL at this point, else unpack_security_parameters() would have failed */
+		ret = tls13_unpack_security_parameters(session, &sb);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
+	}
+
+	if (!session->internals.resumed_security_parameters.pversion->tls13_sem) {
+		ret = _gnutls_hello_ext_unpack(session, &sb);
+		if (ret < 0) {
+			gnutls_assert();
+			goto error;
+		}
 	}
 
 	ret = 0;
@@ -269,7 +302,115 @@ _gnutls_session_unpack(gnutls_session_t session,
 	return ret;
 }
 
+/*
+ * If we're using TLS 1.3 semantics, we might have TLS 1.3-specific data.
+ * Format:
+ *      4 bytes the total length
+ *      4 bytes the ticket lifetime
+ *      4 bytes the ticket age add value
+ *      1 byte the ticket nonce length
+ *      x bytes the ticket nonce
+ *      4 bytes the ticket length
+ *      x bytes the ticket
+ *      1 bytes the resumption master secret length
+ *      x bytes the resumption master secret
+ *     12 bytes the ticket arrival time
+ *      4 bytes the max early data size
+ *
+ * We only store that info if we received a TLS 1.3 NewSessionTicket at some point.
+ * If we didn't receive any NST then we cannot resume a TLS 1.3 session and hence
+ * its nonsense to store all that info.
+ */
+static int
+tls13_pack_security_parameters(gnutls_session_t session, gnutls_buffer_st *ps)
+{
+	int ret = 0;
+	uint32_t length = 0;
+	size_t length_pos;
+	tls13_ticket_st *ticket = &session->internals.tls13_ticket;
 
+	length_pos = ps->length;
+	BUFFER_APPEND_NUM(ps, 0);
+
+	if (ticket->ticket.data != NULL) {
+		BUFFER_APPEND_NUM(ps, ticket->lifetime);
+		length += 4;
+		BUFFER_APPEND_NUM(ps, ticket->age_add);
+		length += 4;
+		BUFFER_APPEND_PFX1(ps,
+				   ticket->nonce,
+				   ticket->nonce_size);
+		length += (1 + ticket->nonce_size);
+		BUFFER_APPEND_PFX4(ps,
+				   ticket->ticket.data,
+				   ticket->ticket.size);
+		length += (4 + ticket->ticket.size);
+		BUFFER_APPEND_PFX1(ps,
+				   ticket->resumption_master_secret,
+				   ticket->prf->output_size);
+		length += (1 + ticket->prf->output_size);
+		BUFFER_APPEND_TS(ps, ticket->arrival_time);
+		length += 12;
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.
+				  max_early_data_size);
+		length += 4;
+
+		/* Overwrite the length field */
+		_gnutls_write_uint32(length, ps->data + length_pos);
+	}
+
+	return ret;
+}
+
+static int
+tls13_unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st *ps)
+{
+	uint32_t ttl_len;
+	tls13_ticket_st *ticket = &session->internals.tls13_ticket;
+	gnutls_datum_t t;
+	int ret = 0;
+
+	BUFFER_POP_NUM(ps, ttl_len);
+
+	if (ttl_len > 0) {
+		BUFFER_POP_NUM(ps, ticket->lifetime);
+		BUFFER_POP_NUM(ps, ticket->age_add);
+
+		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
+		if (ret < 0 || t.size > sizeof(ticket->nonce)) {
+			    ret = GNUTLS_E_PARSING_ERROR;
+			    gnutls_assert();
+			    goto error;
+		}
+		ticket->nonce_size = t.size;
+		memcpy(ticket->nonce, t.data, t.size);
+
+		BUFFER_POP_DATUM(ps, &ticket->ticket);
+
+		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
+		if (ret < 0 || t.size > sizeof(ticket->resumption_master_secret)) {
+			    ret = GNUTLS_E_PARSING_ERROR;
+			    gnutls_assert();
+			    goto error;
+		}
+		memcpy(ticket->resumption_master_secret, t.data, t.size);
+
+		if (unlikely(session->internals.resumed_security_parameters.prf == NULL ||
+		    session->internals.resumed_security_parameters.prf->output_size != t.size))
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+
+		ticket->prf = session->internals.resumed_security_parameters.prf;
+
+		BUFFER_POP_TS(ps, ticket->arrival_time);
+		BUFFER_POP_NUM(ps,
+			       session->security_parameters.
+			       max_early_data_size);
+	}
+
+error:
+	return ret;
+}
 
 /* Format: 
  *      1 byte the credentials type
@@ -317,12 +458,23 @@ pack_certificate_auth_info(gnutls_session_t session, gnutls_buffer_st * ps)
 
 		BUFFER_APPEND_NUM(ps, info->ncerts);
 
-		for (i = 0; i < info->ncerts; i++)
+		for (i = 0; i < info->ncerts; i++) {
 			BUFFER_APPEND_PFX4(ps,
 					   info->raw_certificate_list[i].
 					   data,
 					   info->raw_certificate_list[i].
 					   size);
+		}
+
+		BUFFER_APPEND_NUM(ps, info->nocsp);
+
+		for (i = 0; i < info->nocsp; i++) {
+			BUFFER_APPEND_PFX4(ps,
+					   info->raw_ocsp_list[i].
+					   data,
+					   info->raw_ocsp_list[i].
+					   size);
+		}
 	}
 
 	/* write the real size */
@@ -343,6 +495,8 @@ unpack_certificate_auth_info(gnutls_session_t session,
 	unsigned int i = 0, j = 0;
 	size_t pack_size;
 	cert_auth_info_t info = NULL;
+	unsigned cur_ncerts = 0;
+	unsigned cur_nocsp = 0;
 
 	BUFFER_POP_NUM(ps, pack_size);
 
@@ -352,7 +506,7 @@ unpack_certificate_auth_info(gnutls_session_t session,
 	/* client and server have the same auth_info here
 	 */
 	ret =
-	    _gnutls_auth_info_set(session, GNUTLS_CRD_CERTIFICATE,
+	    _gnutls_auth_info_init(session, GNUTLS_CRD_CERTIFICATE,
 				  sizeof(cert_auth_info_st), 1);
 	if (ret < 0) {
 		gnutls_assert();
@@ -383,6 +537,25 @@ unpack_certificate_auth_info(gnutls_session_t session,
 
 	for (i = 0; i < info->ncerts; i++) {
 		BUFFER_POP_DATUM(ps, &info->raw_certificate_list[i]);
+		cur_ncerts++;
+	}
+
+	/* read OCSP responses */
+	BUFFER_POP_NUM(ps, info->nocsp);
+
+	if (info->nocsp > 0) {
+		info->raw_ocsp_list =
+		    gnutls_calloc(info->nocsp, sizeof(gnutls_datum_t));
+		if (info->raw_ocsp_list == NULL) {
+			gnutls_assert();
+			ret = GNUTLS_E_MEMORY_ERROR;
+			goto error;
+		}
+	}
+
+	for (i = 0; i < info->nocsp; i++) {
+		BUFFER_POP_DATUM(ps, &info->raw_ocsp_list[i]);
+		cur_nocsp++;
 	}
 
 	return 0;
@@ -393,10 +566,14 @@ unpack_certificate_auth_info(gnutls_session_t session,
 		_gnutls_free_datum(&info->dh.generator);
 		_gnutls_free_datum(&info->dh.public_key);
 
-		for (j = 0; j < i; j++)
+		for (j = 0; j < cur_ncerts; j++)
 			_gnutls_free_datum(&info->raw_certificate_list[j]);
 
+		for (j = 0; j < cur_nocsp; j++)
+			_gnutls_free_datum(&info->raw_ocsp_list[j]);
+
 		gnutls_free(info->raw_certificate_list);
+		gnutls_free(info->raw_ocsp_list);
 	}
 
 	return ret;
@@ -455,7 +632,7 @@ unpack_srp_auth_info(gnutls_session_t session, gnutls_buffer_st * ps)
 	}
 
 	ret =
-	    _gnutls_auth_info_set(session, GNUTLS_CRD_SRP,
+	    _gnutls_auth_info_init(session, GNUTLS_CRD_SRP,
 				  sizeof(srp_server_auth_info_st), 1);
 	if (ret < 0) {
 		gnutls_assert();
@@ -537,7 +714,7 @@ unpack_anon_auth_info(gnutls_session_t session, gnutls_buffer_st * ps)
 	/* client and server have the same auth_info here
 	 */
 	ret =
-	    _gnutls_auth_info_set(session, GNUTLS_CRD_ANON,
+	    _gnutls_auth_info_init(session, GNUTLS_CRD_ANON,
 				  sizeof(anon_auth_info_st), 1);
 	if (ret < 0) {
 		gnutls_assert();
@@ -630,7 +807,7 @@ unpack_psk_auth_info(gnutls_session_t session, gnutls_buffer_st * ps)
 	unsigned pack_size;
 
 	ret =
-	    _gnutls_auth_info_set(session, GNUTLS_CRD_PSK,
+	    _gnutls_auth_info_init(session, GNUTLS_CRD_PSK,
 				  sizeof(psk_auth_info_st), 1);
 	if (ret < 0) {
 		gnutls_assert();
@@ -684,38 +861,6 @@ unpack_psk_auth_info(gnutls_session_t session, gnutls_buffer_st * ps)
 
 /* Packs the security parameters.
  */
-
-/* Format: 
- *      4 bytes the total security data size
- *      1 byte the entity type (client/server)
- *      1 byte the key exchange algorithm used
- *      1 byte the read cipher algorithm
- *      1 byte the read mac algorithm
- *      1 byte the read compression algorithm
- *
- *      1 byte the write cipher algorithm
- *      1 byte the write mac algorithm
- *      1 byte the write compression algorithm
- *
- *      1 byte the certificate type
- *      1 byte the protocol version
- *
- *      2 bytes the cipher suite
- *
- *      48 bytes the master secret
- *
- *      32 bytes the client random
- *      32 bytes the server random
- *
- *      1 byte the session ID size
- *      x bytes the session ID (32 bytes max)
- *
- *      4 bytes the new record padding flag
- *      4 bytes the ECC curve
- *	    -------------------
- *		MAX: 169 bytes
- *
- */
 static int
 pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 {
@@ -723,15 +868,15 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	int ret;
 	int size_offset;
 	size_t cur_size;
-	record_parameters_st *params;
 
 	if (session->security_parameters.epoch_read
-	    != session->security_parameters.epoch_write) {
+	    != session->security_parameters.epoch_write &&
+	    !(session->internals.hsk_flags & HSK_EARLY_START_USED)) {
 		gnutls_assert();
 		return GNUTLS_E_UNAVAILABLE_DURING_HANDSHAKE;
 	}
 
-	ret = _gnutls_epoch_get(session, EPOCH_READ_CURRENT, &params);
+	ret = _gnutls_epoch_get(session, EPOCH_READ_CURRENT, NULL);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -743,43 +888,68 @@ pack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	BUFFER_APPEND_NUM(ps, 0);
 	cur_size = ps->length;
 
-
 	BUFFER_APPEND_NUM(ps, session->security_parameters.entity);
-	BUFFER_APPEND_NUM(ps, session->security_parameters.kx_algorithm);
-	BUFFER_APPEND(ps, session->security_parameters.cipher_suite, 2);
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.compression_method);
-	BUFFER_APPEND_NUM(ps, session->security_parameters.cert_type);
-	BUFFER_APPEND_NUM(ps, session->security_parameters.pversion->id);
+	BUFFER_APPEND_NUM(ps, session->security_parameters.prf->id);
 
-	BUFFER_APPEND(ps, session->security_parameters.master_secret,
-		      GNUTLS_MASTER_SIZE);
-	BUFFER_APPEND(ps, session->security_parameters.client_random,
-		      GNUTLS_RANDOM_SIZE);
-	BUFFER_APPEND(ps, session->security_parameters.server_random,
-		      GNUTLS_RANDOM_SIZE);
+	BUFFER_APPEND_NUM(ps,
+			  session->security_parameters.client_auth_type);
+	BUFFER_APPEND_NUM(ps,
+			  session->security_parameters.server_auth_type);
 
 	BUFFER_APPEND(ps, &session->security_parameters.session_id_size,
 		      1);
 	BUFFER_APPEND(ps, session->security_parameters.session_id,
 		      session->security_parameters.session_id_size);
 
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.
-			  max_record_send_size);
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.
-			  max_record_recv_size);
-	BUFFER_APPEND_NUM(ps, session->security_parameters.ecc_curve);
+	BUFFER_APPEND_NUM(ps, session->security_parameters.pversion->id);
+
+	BUFFER_APPEND_NUM(ps, session->security_parameters.client_ctype);
+	BUFFER_APPEND_NUM(ps, session->security_parameters.server_ctype);
+
+	/* if we are under TLS 1.3 do not pack keys or params negotiated using an extension
+	 * they are not necessary */
+	if (!session->security_parameters.pversion->tls13_sem) {
+		BUFFER_APPEND(ps, session->security_parameters.cs->id, 2);
+
+		BUFFER_APPEND_PFX1(ps, session->security_parameters.master_secret,
+			      GNUTLS_MASTER_SIZE);
+		BUFFER_APPEND_PFX1(ps, session->security_parameters.client_random,
+			      GNUTLS_RANDOM_SIZE);
+		BUFFER_APPEND_PFX1(ps, session->security_parameters.server_random,
+			      GNUTLS_RANDOM_SIZE);
+
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.
+				  max_record_send_size);
+
+		/* reset max_record_recv_size if it was negotiated
+		 * using the record_size_limit extension */
+		if (session->internals.hsk_flags & HSK_RECORD_SIZE_LIMIT_NEGOTIATED) {
+			BUFFER_APPEND_NUM(ps,
+					  session->security_parameters.
+					  max_record_send_size);
+		} else {
+			BUFFER_APPEND_NUM(ps,
+					  session->security_parameters.
+					  max_record_recv_size);
+		}
+
+		if (session->security_parameters.grp) {
+			BUFFER_APPEND_NUM(ps, session->security_parameters.grp->id);
+		} else {
+			BUFFER_APPEND_NUM(ps, 0);
+		}
+
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.server_sign_algo);
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.client_sign_algo);
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.ext_master_secret);
+		BUFFER_APPEND_NUM(ps,
+				  session->security_parameters.etm);
+	}
 
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.server_sign_algo);
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.client_sign_algo);
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.ext_master_secret);
-	BUFFER_APPEND_NUM(ps,
-			  session->security_parameters.etm);
 
 	_gnutls_write_uint32(ps->length - cur_size,
 			     ps->data + size_offset);
@@ -793,7 +963,9 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	size_t pack_size;
 	int ret;
 	unsigned version;
+	gnutls_datum_t t;
 	time_t timestamp;
+	uint8_t cs[2];
 
 	BUFFER_POP_NUM(ps, pack_size);
 
@@ -807,40 +979,22 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 	session->internals.resumed_security_parameters.timestamp =
 	    timestamp;
 
-	timestamp = gnutls_time(0);
-
 	BUFFER_POP_NUM(ps,
 		       session->internals.resumed_security_parameters.
 		       entity);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       kx_algorithm);
-	BUFFER_POP(ps,
-		   session->internals.resumed_security_parameters.
-		   cipher_suite, 2);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       compression_method);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       cert_type);
+
 	BUFFER_POP_NUM(ps, version);
-	session->internals.resumed_security_parameters.pversion =
-	    version_to_entry(version);
-	if (session->internals.resumed_security_parameters.pversion ==
-	    NULL)
+	session->internals.resumed_security_parameters.prf = mac_to_entry(version);
+	if (session->internals.resumed_security_parameters.prf == NULL)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	BUFFER_POP(ps,
-		   session->internals.resumed_security_parameters.
-		   master_secret, GNUTLS_MASTER_SIZE);
+	BUFFER_POP_NUM(ps,
+		       session->internals.resumed_security_parameters.
+		       client_auth_type);
+	BUFFER_POP_NUM(ps,
+		       session->internals.resumed_security_parameters.
+		       server_auth_type);
 
-	BUFFER_POP(ps,
-		   session->internals.resumed_security_parameters.
-		   client_random, GNUTLS_RANDOM_SIZE);
-	BUFFER_POP(ps,
-		   session->internals.resumed_security_parameters.
-		   server_random, GNUTLS_RANDOM_SIZE);
 	BUFFER_POP(ps,
 		   &session->internals.resumed_security_parameters.
 		   session_id_size, 1);
@@ -851,43 +1005,87 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
 		   session->internals.resumed_security_parameters.
 		   session_id_size);
 
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       max_record_send_size);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       max_record_recv_size);
+	BUFFER_POP_NUM(ps, version);
+	session->internals.resumed_security_parameters.pversion =
+	    version_to_entry(version);
+	if (session->internals.resumed_security_parameters.pversion ==
+	    NULL)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
 	BUFFER_POP_NUM(ps,
 		       session->internals.resumed_security_parameters.
-		       ecc_curve);
+		       client_ctype);
 	BUFFER_POP_NUM(ps,
 		       session->internals.resumed_security_parameters.
-		       server_sign_algo);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       client_sign_algo);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       ext_master_secret);
-	BUFFER_POP_NUM(ps,
-		       session->internals.resumed_security_parameters.
-		       etm);
+		       server_ctype);
 
-	if (session->internals.resumed_security_parameters.
-	    max_record_recv_size == 0
-	    || session->internals.resumed_security_parameters.
-	    max_record_send_size == 0) {
-		return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
-	}
+	if (!session->internals.resumed_security_parameters.pversion->tls13_sem) {
+		BUFFER_POP(ps, cs, 2);
+		session->internals.resumed_security_parameters.cs = ciphersuite_to_entry(cs);
+		if (session->internals.resumed_security_parameters.cs == NULL)
+			return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
-	if (timestamp -
-	    session->internals.resumed_security_parameters.timestamp >
-	    session->internals.expire_time
-	    || session->internals.resumed_security_parameters.timestamp >
-	    timestamp) {
-		gnutls_assert();
-		return GNUTLS_E_EXPIRED;
+		/* master secret */
+		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
+		if (ret < 0) {
+			ret = GNUTLS_E_PARSING_ERROR;
+			gnutls_assert();
+			goto error;
+		}
+		if (t.size == GNUTLS_MASTER_SIZE)
+			memcpy(session->internals.resumed_security_parameters.master_secret, t.data, t.size);
+
+		/* client random */
+		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
+		if (ret < 0) {
+			ret = GNUTLS_E_PARSING_ERROR;
+			gnutls_assert();
+			goto error;
+		}
+		if (t.size == GNUTLS_RANDOM_SIZE)
+			memcpy(session->internals.resumed_security_parameters.client_random, t.data, t.size);
+
+		/* server random */
+		ret = _gnutls_buffer_pop_datum_prefix8(ps, &t);
+		if (ret < 0) {
+			ret = GNUTLS_E_PARSING_ERROR;
+			gnutls_assert();
+			goto error;
+		}
+		if (t.size == GNUTLS_RANDOM_SIZE)
+			memcpy(session->internals.resumed_security_parameters.server_random, t.data, t.size);
+
+
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       max_record_send_size);
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       max_record_recv_size);
+
+		BUFFER_POP_NUM(ps, ret);
+		session->internals.resumed_security_parameters.grp = _gnutls_id_to_group(ret);
+		/* it can be null */
+
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       server_sign_algo);
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       client_sign_algo);
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       ext_master_secret);
+		BUFFER_POP_NUM(ps,
+			       session->internals.resumed_security_parameters.
+			       etm);
+
+		if (session->internals.resumed_security_parameters.
+		    max_record_recv_size == 0
+		    || session->internals.resumed_security_parameters.
+		    max_record_send_size == 0) {
+			return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
+		}
 	}
 
 	ret = 0;
@@ -904,7 +1102,7 @@ unpack_security_parameters(gnutls_session_t session, gnutls_buffer_st * ps)
  * @kx: the key exchange method
  * @cipher: the cipher
  * @mac: the MAC algorithm
- * @comp: the compression method
+ * @comp: the compression method (ignored)
  * @master: the master key to use
  * @session_id: the session identifier
  *
@@ -927,32 +1125,39 @@ gnutls_session_set_premaster(gnutls_session_t session, unsigned int entity,
 			     const gnutls_datum_t * session_id)
 {
 	int ret;
+	uint8_t cs[2];
 
 	memset(&session->internals.resumed_security_parameters, 0,
 	       sizeof(session->internals.resumed_security_parameters));
 
 	session->internals.resumed_security_parameters.entity = entity;
-	session->internals.resumed_security_parameters.kx_algorithm = kx;
 
 	ret =
-	    _gnutls_cipher_suite_get_id(kx, cipher, mac,
-					session->internals.
-					resumed_security_parameters.
-					cipher_suite);
+	    _gnutls_cipher_suite_get_id(kx, cipher, mac, cs);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
 
-	session->internals.resumed_security_parameters.compression_method =
-	    comp;
-	session->internals.resumed_security_parameters.cert_type =
-	    DEFAULT_CERT_TYPE;
+	session->internals.resumed_security_parameters.cs = ciphersuite_to_entry(cs);
+	if (session->internals.resumed_security_parameters.cs == NULL)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+	session->internals.resumed_security_parameters.client_ctype =
+			DEFAULT_CERT_TYPE;
+	session->internals.resumed_security_parameters.server_ctype =
+			DEFAULT_CERT_TYPE;
 	session->internals.resumed_security_parameters.pversion =
 	    version_to_entry(version);
-
 	if (session->internals.resumed_security_parameters.pversion ==
 	    NULL)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
+	if (session->internals.resumed_security_parameters.pversion->selectable_prf)
+		session->internals.resumed_security_parameters.prf = mac_to_entry(session->internals.resumed_security_parameters.cs->prf);
+	else
+		session->internals.resumed_security_parameters.prf = mac_to_entry(GNUTLS_MAC_MD5_SHA1);
+	if (session->internals.resumed_security_parameters.prf == NULL)
+		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
 	if (master->size != GNUTLS_MASTER_SIZE)
 		return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 
@@ -975,8 +1180,9 @@ gnutls_session_set_premaster(gnutls_session_t session, unsigned int entity,
 	session->internals.resumed_security_parameters.timestamp =
 	    gnutls_time(0);
 
-	session->internals.resumed_security_parameters.ecc_curve =
-	    GNUTLS_ECC_CURVE_INVALID;
+	session->internals.resumed_security_parameters.grp = 0;
+
+	session->internals.resumed_security_parameters.post_handshake_auth = 0;
 
 	session->internals.premaster_set = 1;
 
