@@ -27,37 +27,14 @@
 #include "mem.h"
 #include "mbuffers.h"
 #include "secrets.h"
-#include "system/ktls.h"
 
-#define KEY_UPDATES_WINDOW 1000
-#define KEY_UPDATES_PER_WINDOW 8
-
-/*
- * Sets kTLS keys if enabled.
- * If this operation fails with GNUTLS_E_INTERNAL_ERROR, KTLS is disabled
- * because KTLS most likely doesn't support key update.
- */
-static inline int set_ktls_keys(gnutls_session_t session,
-				gnutls_transport_ktls_enable_flags_t iface)
-{
-	if (_gnutls_ktls_set_keys(session, iface) < 0) {
-		session->internals.ktls_enabled = 0;
-		session->internals.invalid_connection = true;
-		session->internals.resumable = false;
-		_gnutls_audit_log(
-			session,
-			"invalidating session: KTLS - couldn't update keys\n");
-		return GNUTLS_E_INTERNAL_ERROR;
-	}
-	return 0;
-}
+#define KEY_UPDATES_PER_SEC 1
 
 static int update_keys(gnutls_session_t session, hs_stage_t stage)
 {
 	int ret;
 
-	ret = _tls13_update_secret(session,
-				   session->key.proto.tls13.temp_secret,
+	ret = _tls13_update_secret(session, session->key.proto.tls13.temp_secret,
 				   session->key.proto.tls13.temp_secret_size);
 	if (ret < 0)
 		return gnutls_assert_val(ret);
@@ -71,22 +48,8 @@ static int update_keys(gnutls_session_t session, hs_stage_t stage)
 	 * write keys */
 	if (session->internals.recv_state == RECV_STATE_EARLY_START) {
 		ret = _tls13_write_connection_state_init(session, stage);
-		if (ret < 0)
-			return gnutls_assert_val(ret);
-
-		if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND))
-			ret = set_ktls_keys(session, GNUTLS_KTLS_SEND);
 	} else {
 		ret = _tls13_connection_state_init(session, stage);
-		if (ret < 0)
-			return gnutls_assert_val(ret);
-
-		if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_SEND) &&
-		    stage == STAGE_UPD_OURS)
-			ret = set_ktls_keys(session, GNUTLS_KTLS_SEND);
-		else if (IS_KTLS_ENABLED(session, GNUTLS_KTLS_RECV) &&
-			 stage == STAGE_UPD_PEERS)
-			ret = set_ktls_keys(session, GNUTLS_KTLS_RECV);
 	}
 	if (ret < 0)
 		return gnutls_assert_val(ret);
@@ -97,35 +60,25 @@ static int update_keys(gnutls_session_t session, hs_stage_t stage)
 int _gnutls13_recv_key_update(gnutls_session_t session, gnutls_buffer_st *buf)
 {
 	int ret;
-	struct timespec now;
+	time_t now = gnutls_time(0);
 
 	if (buf->length != 1)
 		return gnutls_assert_val(GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-	gnutls_gettime(&now);
-
-	/* Roll over the counter if the time window has elapsed */
-	if (session->internals.key_update_count == 0 ||
-	    timespec_sub_ms(&now, &session->internals.last_key_update) >
-		    KEY_UPDATES_WINDOW) {
-		session->internals.last_key_update = now;
-		session->internals.key_update_count = 0;
-	}
-
-	if (unlikely(++session->internals.key_update_count >
-		     KEY_UPDATES_PER_WINDOW)) {
-		_gnutls_debug_log(
-			"reached maximum number of key updates per %d milliseconds (%d)\n",
-			KEY_UPDATES_WINDOW, KEY_UPDATES_PER_WINDOW);
+	if (unlikely(now - session->internals.last_key_update < KEY_UPDATES_PER_SEC)) {
+		_gnutls_debug_log("reached maximum number of key updates per second (%d)\n",
+				  KEY_UPDATES_PER_SEC);
 		return gnutls_assert_val(GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS);
 	}
 
+	session->internals.last_key_update = now;
+
 	_gnutls_epoch_gc(session);
 
 	_gnutls_handshake_log("HSK[%p]: received TLS 1.3 key update (%u)\n",
 			      session, (unsigned)buf->data[0]);
 
-	switch (buf->data[0]) {
+	switch(buf->data[0]) {
 	case 0:
 		/* peer updated its key, not requested our key update */
 		ret = update_keys(session, STAGE_UPD_PEERS);
@@ -150,11 +103,11 @@ int _gnutls13_recv_key_update(gnutls_session_t session, gnutls_buffer_st *buf)
 		 * message.
 		 */
 		if (session->internals.rsend_state == RECORD_SEND_NORMAL)
-			session->internals.rsend_state =
-				RECORD_SEND_KEY_UPDATE_1;
+			session->internals.rsend_state = RECORD_SEND_KEY_UPDATE_1;
 		else if (session->internals.rsend_state == RECORD_SEND_CORKED)
-			session->internals.rsend_state =
-				RECORD_SEND_CORKED_TO_KU;
+			session->internals.rsend_state = RECORD_SEND_CORKED_TO_KU;
+		else
+			return gnutls_assert_val(GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
 
 		break;
 	default:
@@ -166,8 +119,7 @@ int _gnutls13_recv_key_update(gnutls_session_t session, gnutls_buffer_st *buf)
 	return 0;
 }
 
-int _gnutls13_send_key_update(gnutls_session_t session, unsigned again,
-			      unsigned flags /* GNUTLS_KU_* */)
+int _gnutls13_send_key_update(gnutls_session_t session, unsigned again, unsigned flags /* GNUTLS_KU_* */)
 {
 	int ret;
 	mbuffer_st *bufel = NULL;
@@ -183,8 +135,7 @@ int _gnutls13_send_key_update(gnutls_session_t session, unsigned again,
 			val = 0x00;
 		}
 
-		_gnutls_handshake_log("HSK[%p]: sending key update (%u)\n",
-				      session, (unsigned)val);
+		_gnutls_handshake_log("HSK[%p]: sending key update (%u)\n", session, (unsigned)val);
 
 		bufel = _gnutls_handshake_alloc(session, 1);
 		if (bufel == NULL)
@@ -196,10 +147,10 @@ int _gnutls13_send_key_update(gnutls_session_t session, unsigned again,
 			gnutls_assert();
 			goto cleanup;
 		}
+
 	}
 
-	return _gnutls_send_handshake(session, bufel,
-				      GNUTLS_HANDSHAKE_KEY_UPDATE);
+	return _gnutls_send_handshake(session, bufel, GNUTLS_HANDSHAKE_KEY_UPDATE);
 
 cleanup:
 	_mbuffer_xfree(&bufel);
@@ -236,7 +187,8 @@ int gnutls_session_key_update(gnutls_session_t session, unsigned flags)
 	if (!vers->tls13_sem)
 		return GNUTLS_E_INVALID_REQUEST;
 
-	ret = _gnutls13_send_key_update(session, AGAIN(STATE150), flags);
+	ret =
+	    _gnutls13_send_key_update(session, AGAIN(STATE150), flags);
 	STATE = STATE150;
 
 	if (ret < 0) {
