@@ -17,7 +17,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>
  *
  */
 
@@ -33,10 +33,9 @@
 #include "x509_int.h"
 #include "extras/hex.h"
 #include <common.h>
-#include <c-ctype.h>
 
 /* TIME functions 
- * Convertions between generalized or UTC time to time_t
+ * Conversions between generalized or UTC time to time_t
  *
  */
 
@@ -65,12 +64,12 @@ static const int MONTHDAYS[] = {
 
     /* Whether a given year is a leap year. */
 #define ISLEAP(year) \
-        (((year) % 4) == 0 && (((year) % 100) != 0 || ((year) % 400) == 0))
+	(((year) % 4) == 0 && (((year) % 100) != 0 || ((year) % 400) == 0))
 
 /*
  **  Given a struct tm representing a calendar time in UTC, convert it to
  **  seconds since epoch.  Returns (time_t) -1 if the time is not
- **  convertable.  Note that this function does not canonicalize the provided
+ **  convertible.  Note that this function does not canonicalize the provided
  **  struct tm, nor does it allow out of range values or years before 1970.
  */
 static time_t mktime_utc(const struct fake_tm *tm)
@@ -86,6 +85,10 @@ static time_t mktime_utc(const struct fake_tm *tm)
 	if (tm->tm_mon < 0 || tm->tm_mon > 11 || tm->tm_year < 1970)
 		return (time_t) - 1;
 
+	/* Check for "obvious" mistakes in dates */
+	if (tm->tm_sec > 60 || tm->tm_min > 59 || tm->tm_mday > 31 || tm->tm_mday < 1 || tm->tm_hour > 23)
+		return (time_t) - 1;
+
 /* Convert to a time_t. 
  */
 	for (i = 1970; i < tm->tm_year; i++)
@@ -208,8 +211,13 @@ time_t _gnutls_x509_generalTime2gtime(const char *ttime)
 
 	if (strchr(ttime, 'Z') == 0) {
 		gnutls_assert();
-		/* sorry we don't support it yet
-		 */
+		/* required to be in GMT */
+		return (time_t) - 1;
+	}
+
+	if (strchr(ttime, '.') != 0) {
+		gnutls_assert();
+		/* no fractional seconds allowed */
 		return (time_t) - 1;
 	}
 	xx[4] = 0;
@@ -235,10 +243,10 @@ gtime_to_suitable_time(time_t gtime, char *str_time, size_t str_time_size, unsig
 		|| gtime >= 253402210800
 #endif
 	 ) {
-        	if (tag)
-        		*tag = ASN1_TAG_GENERALIZEDTime;
-        	snprintf(str_time, str_time_size, "99991231235959Z");
-        	return 0;
+		if (tag)
+			*tag = ASN1_TAG_GENERALIZEDTime;
+		snprintf(str_time, str_time_size, "99991231235959Z");
+		return 0;
 	}
 
 	if (!gmtime_r(&gtime, &_tm)) {
@@ -248,11 +256,11 @@ gtime_to_suitable_time(time_t gtime, char *str_time, size_t str_time_size, unsig
 
 	if (_tm.tm_year >= 150) {
 		if (tag)
-        		*tag = ASN1_TAG_GENERALIZEDTime;
+			*tag = ASN1_TAG_GENERALIZEDTime;
 		ret = strftime(str_time, str_time_size, "%Y%m%d%H%M%SZ", &_tm);
 	} else {
 		if (tag)
-        		*tag = ASN1_TAG_UTCTime;
+			*tag = ASN1_TAG_UTCTime;
 		ret = strftime(str_time, str_time_size, "%y%m%d%H%M%SZ", &_tm);
 	}
 	if (!ret) {
@@ -274,8 +282,8 @@ gtime_to_generalTime(time_t gtime, char *str_time, size_t str_time_size)
 		|| gtime >= 253402210800
 #endif
 	 ) {
-        	snprintf(str_time, str_time_size, "99991231235959Z");
-        	return 0;
+		snprintf(str_time, str_time_size, "99991231235959Z");
+		return 0;
 	}
 
 	if (!gmtime_r(&gtime, &_tm)) {
