@@ -16,7 +16,8 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with GnuTLS.  If not, see <https://www.gnu.org/licenses/>.
+ * along with GnuTLS; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +29,7 @@
 
 #if defined(_WIN32)
 
-int main(void)
+int main()
 {
 	exit(77);
 }
@@ -64,6 +65,7 @@ static void client_log_func(int level, const char *str)
 	fprintf(stderr, "client|<%d>| %s", level, str);
 }
 
+
 #define MAX_BUF 1024
 
 static void client(int fd)
@@ -88,13 +90,11 @@ static void client(int fd)
 
 	/* Initialize TLS session
 	 */
-	gnutls_init(&session, GNUTLS_CLIENT | GNUTLS_AUTO_REAUTH);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_init(&session, GNUTLS_CLIENT|GNUTLS_AUTO_REAUTH);
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 
 	/* Use default priorities */
-	gnutls_priority_set_direct(
-		session, "NORMAL:-VERS-TLS-ALL:+VERS-TLS1.1:+VERS-TLS1.2",
-		NULL);
+	gnutls_priority_set_direct(session, "NORMAL:-VERS-TLS-ALL:+VERS-TLS1.1:+VERS-TLS1.2", NULL);
 
 	/* put the anonymous credentials to the current session
 	 */
@@ -107,7 +107,8 @@ static void client(int fd)
 	 */
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 
 	if (ret < 0) {
 		fail("client: Handshake failed\n");
@@ -120,12 +121,15 @@ static void client(int fd)
 
 	if (debug)
 		success("client: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	do {
-		ret = gnutls_record_recv(session, buffer, MAX_BUF);
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+		ret =
+		    gnutls_record_recv(session, buffer,
+					MAX_BUF);
+	} while (ret == GNUTLS_E_AGAIN
+		 || ret == GNUTLS_E_INTERRUPTED);
 
 	assert(ret == 0);
 
@@ -168,14 +172,12 @@ static void server(int fd)
 	gnutls_anon_allocate_server_credentials(&anoncred);
 
 	gnutls_init(&session, GNUTLS_SERVER);
-	gnutls_handshake_set_timeout(session, get_timeout());
+	gnutls_handshake_set_timeout(session, 20 * 1000);
 
 	/* avoid calling all the priority functions, since the defaults
 	 * are adequate.
 	 */
-	gnutls_priority_set_direct(
-		session, "NORMAL:-VERS-TLS-ALL:+VERS-TLS1.1:+VERS-TLS1.2",
-		NULL);
+	gnutls_priority_set_direct(session, "NORMAL:-VERS-TLS-ALL:+VERS-TLS1.1:+VERS-TLS1.2", NULL);
 
 	gnutls_credentials_set(session, GNUTLS_CRD_ANON, anoncred);
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, x509_cred);
@@ -184,7 +186,8 @@ static void server(int fd)
 
 	do {
 		ret = gnutls_handshake(session);
-	} while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
+	}
+	while (ret < 0 && gnutls_error_is_fatal(ret) == 0);
 	if (ret < 0) {
 		close(fd);
 		gnutls_deinit(session);
@@ -196,18 +199,18 @@ static void server(int fd)
 
 	if (debug)
 		success("server: TLS version is: %s\n",
-			gnutls_protocol_get_name(
-				gnutls_protocol_get_version(session)));
+			gnutls_protocol_get_name
+			(gnutls_protocol_get_version(session)));
 
 	if (debug)
 		success("server: sending rehandshake request\n");
 
 	do {
 		ret = gnutls_rehandshake(session);
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	} while(ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 	if (ret < 0) {
-		fail("Error sending %d byte packet: %s\n", (int)sizeof(buffer),
-		     gnutls_strerror(ret));
+		fail("Error sending %d byte packet: %s\n",
+		     (int)sizeof(buffer), gnutls_strerror(ret));
 	}
 
 	if (debug)
@@ -215,8 +218,7 @@ static void server(int fd)
 
 	ret = gnutls_handshake(session);
 	if (ret < 0) {
-		fail("server: didn't complete handshake: %s\n",
-		     gnutls_strerror(ret));
+		fail("server: didn't complete handshake: %s\n", gnutls_strerror(ret));
 	}
 
 	if (debug)
@@ -224,7 +226,7 @@ static void server(int fd)
 
 	do {
 		ret = gnutls_bye(session, GNUTLS_SHUT_RDWR);
-	} while (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
+	} while(ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED);
 
 	close(fd);
 	gnutls_deinit(session);
@@ -273,4 +275,4 @@ void doit(void)
 	}
 }
 
-#endif /* _WIN32 */
+#endif				/* _WIN32 */
